//==============================================
//      File rbdeutils.pas
//
//      Components library for Delphi and BCB++.
//      , ,   
//           BDE.
//      Procedures, functions, constants and types
//      for working with database (BDE).
//
//      Copyright 1998-2002 Polaris Software
//      http://polesoft.da.ru
//      mailto: PolarisLib@mail.ru
//==============================================
unit rBDEUtils;

{$I POLARIS.INC}

interface

uses rDBConst, BDE, DBTables, DB, SysUtils, rUtils, ErrorMes;

const
{$IFDEF POLARIS_D4}
  DBMasks: array[TTableType] of string[5] = ('', '*.DB', '*.DBF', '*.TXT', '*.DBF');
{$ELSE}
  DBMasks: array[TTableType] of string[5] = ('', '*.DB', '*.DBF', '*.TXT');
{$ENDIF}

//  SQL 
// Execute SQL
procedure ExecuteSQL(ADatabase: TDatabase; SQL: string);

//  /   
// System date&time on DB server
function DBSysDate(ADatabase: TDatabase; WithTime: Boolean): TDateTime;

{$IFNDEF POLARIS_D4}
//   
// Assign parameter (TParam) value
procedure AssignParam(Param: TParam; Value: Variant);

//    
// Clear all TParam's values
procedure ClearParams(Params: TParams);

//     
// Assign parameter (TParam) value from field
procedure AssignParamFromField(Param: TParam; Field: TField);
{$ENDIF}

//     
// Execute stored procedure within transaction
procedure ExecStoredProc(StoredProc: TStoredProc; DB: TDatabase);

//  
// User name
function GetUser(DB: TDatabase): string;

//   BDE
// BDE errors handler
procedure BDEErrorEHProc(E: Exception; Handler: TErrorHandler);

implementation

procedure ExecuteSQL(ADatabase: TDatabase; SQL: string);
var
  FStmtHandle: hDBIStmt;
begin
  Check(DbiQAlloc(ADataBase.Handle, qrylangSQL, FStmtHandle));
  try
    Check(DbiQPrepare(FStmtHandle, PChar(SQL)));
  except
    DbiQFree(FStmtHandle);
    FStmtHandle := nil;
    raise;
  end;
  Check(DbiQExec(FStmtHandle, nil));
  Check(DbiQFree(FStmtHandle));
end;

function DBSysDate(ADatabase: TDatabase; WithTime: Boolean): TDateTime;
var
  driver: string;
begin
  if ADatabase.AliasName <> '' then
    driver := ADatabase.Session.GetAliasDriverName(ADatabase.AliasName)
  else
    driver := ADatabase.DriverName;
  if driver = 'INTRBASE' then
    with TQuery.Create(nil) do
      try
        DatabaseName := ADatabase.DatabaseName;
        SQL.Text := srSQLIBSysDate;
        Open;
        Result := Fields[0].AsDateTime;
        if not WithTime then
          Result := StrToDate(DateToStr(Result));
        Close;
      finally
        Free;
      end
  else
    if WithTime then Result := SysUtils.Now else Result := SysUtils.Date;
end;

procedure ExecStoredProc(StoredProc: TStoredProc; DB: TDatabase);
var
  b: Boolean;
begin
  with StoredProc, DB do begin
    b := InTransaction;
    if not b then StartTransaction;
    try
      Prepare;
      ExecProc;
      if not b then Commit;
    except
      if not b then Rollback;
      raise;
    end;
  end;
end;

function GetUser(DB: TDatabase): string;
begin
  if Assigned(DB) and DB.Connected then
    Result := DB.Params.Values[szUSERNAME]+' '+srDBUserName
  else
    Result := GetOSUser+' '+srOSUserName;
end;

{$IFNDEF POLARIS_D4}
procedure AssignParam(Param: TParam; Value: Variant);
begin
  if VarIsNull(Value) then Param.Clear else Param.Value := Value;
  Param.Bound := True;
end;

procedure ClearParams(Params: TParams);
var
  I: Integer;
begin
  for I := 0 to Params.Count - 1 do Params[I].Clear;
end;

procedure AssignParamFromField(Param: TParam; Field: TField);
begin
  if Field.IsNull then Param.Clear else Param.Value := Field.Value;
  Param.Bound := True;
end;
{$ENDIF}

procedure BDEErrorEHProc(E: Exception; Handler: TErrorHandler);
var
  i: Integer;
  mes: string;
begin
  with Handler do begin
    TitleMessage := srBDEErrorTitle;
    with E as EDBEngineError do begin
      for i:=0 to ErrorCount-1 do begin  { loop by errors }
        mes := Errors[i].Message;
        // delete last LF or CR in message
        while (Length(mes) > 0) and (Copy(mes, Length(mes), 1)[1] in [#10,#13]) do
          System.Delete(mes, Length(mes), 1);
        if Length(mes) = 0 then Continue;
        if (Handler.Message = '') and Assigned(FIniFile) and
           ((i = ErrorCount-1) or (Errors[i].NativeError <> 0)) then begin
          Handler.Message := ConvertMessage(Errors[i].Message);
          if Handler.Message = '' then Handler.Message := Errors[i].Message;
        end;
        with Errors[i] do
          if NativeError = 0 then
            Description.Add(Format(srBDEErrorDesc,
                                    [ErrorCode, Category, SubCode,mes]))
          else
            Description.Add(Format(srSQLErrorDesc, [NativeError, mes]));
      end;
      if Handler.Message = '' then begin
        Handler.Message := Errors[ErrorCount-1].Message;
        SaveLog := True;
      end;
      // list of active databases
      mes := '';
      for i:=0 to Session.DatabaseCount-1 do
        if Session.Databases[i].Connected then
          if Session.Databases[i].AliasName = '' then
            mes := mes + Session.Databases[i].DatabaseName + ';'
          else
            mes := mes + Session.Databases[i].AliasName + ';';
      if Length(mes) > 0 then
        Description.Add(Format(srActiveDB, [Copy(mes, 1, Length(mes)-1)]));
    end;
  end;
end;

initialization
  AddError('EDBEngineError',BDEErrorEHProc);
end.

