//==============================================
//      File rDLLloader.pas
//
//      Components library for Delphi and BCB++.
//         DLL.
//      Loader of DLL library.
//
//      Copyright 2000-02 Polaris Software
//      http://polesoft.da.ru
//      mailto: PolarisLib@mail.ru
//==============================================
unit rDLLloader;

{$I POLARIS.INC}

interface

uses
  Windows, Messages, SysUtils, Classes;

type
  TrDLLloader = class(TComponent)
  private
    FActive: Boolean;
    FHInstance: HMODULE;
    FLibraryName: TFileName;
    FRequiredProcs: TStrings;
    FVersionInfo: TStrings;
    FAfterLoadLibrary: TNotifyEvent;
    FBeforeFreeLibrary: TNotifyEvent;
    FRequiredVersion: String;
    FFileVersion: String;
    procedure ClearVersionInfo;
    function GetEntryPoint(const Name: String): Pointer;
    procedure InternalGetVersionInfo;
    procedure SetVersionInfo(const Value: TStrings);
    procedure SetLibraryName(const Value: TFileName);
    procedure SetRequiredProcs(const Value: TStrings);
    procedure SetRequiredVersion(const Value: String);
  protected
    procedure CheckRequired;
    procedure DoAfterLoadLibrary; virtual;
    procedure DoBeforeFreeLibrary; virtual;
    procedure FreeDLL;
    property HInstance: HMODULE read FHInstance write FHInstance;
    procedure LoadDLL;
    procedure Loaded; override;
    procedure SetActive(const Value: Boolean); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Close;
    destructor Destroy; override;
    property EntryPoint[const Name: String]: Pointer read GetEntryPoint; default;
    procedure Open;
    property FileVersion: string read FFileVersion;
  published
    property Active: Boolean read FActive write SetActive;
    property LibraryName: TFileName read FLibraryName write SetLibraryName;
    property RequiredProcs: TStrings read FRequiredProcs write SetRequiredProcs;
    property RequiredVersion: String read FRequiredVersion write SetRequiredVersion;
    property VersionInfo: TStrings read FVersionInfo write SetVersionInfo stored False;
    property AfterLoadLibrary: TNotifyEvent read FAfterLoadLibrary write FAfterLoadLibrary;
    property BeforeFreeLibrary: TNotifyEvent read FBeforeFreeLibrary write FBeforeFreeLibrary;
  end;

implementation

uses
  rConst, rUtils;

{ TrDLLloader }

constructor TrDLLloader.Create(AOwner: TComponent);
begin
  inherited;
  FActive := False;
  FHInstance := 0;
  FLibraryName := '';
  FFileVersion := '';
  FRequiredProcs := TStringList.Create;
  FRequiredVersion := '';
  FVersionInfo := TStringList.Create;
  FAfterLoadLibrary := nil;
  FBeforeFreeLibrary := nil;
end;

destructor TrDLLloader.Destroy;
begin
  FreeDLL;
  FVersionInfo.Free;
  FRequiredProcs.Free;
  inherited;
end;

procedure TrDLLloader.CheckRequired;
var
  i: Integer;
  hProc: Pointer;
begin
  for i := 0 to FRequiredProcs.Count-1 do begin
    hProc := EntryPoint[FRequiredProcs[i]];
    if not Assigned(hProc) then
      raise Exception.CreateFmt(srDLLCheckRequiredError,[FRequiredProcs[i]]);
  end;
end;

procedure TrDLLloader.DoAfterLoadLibrary;
begin
  if Assigned(FAfterLoadLibrary) then
    FAfterLoadLibrary(Self);
end;

procedure TrDLLloader.DoBeforeFreeLibrary;
begin
  if Assigned(FBeforeFreeLibrary) then
    FBeforeFreeLibrary(Self);
end;

procedure TrDLLloader.FreeDLL;
begin
  if (FHInstance <> 0) then begin
    DoBeforeFreeLibrary;
    ClearVersionInfo;
    FreeLibrary(FHInstance);
    FHInstance := 0;
  end;
end;

procedure TrDLLloader.ClearVersionInfo;
begin
  FFileVersion := '';
  FVersionInfo.Clear;
end;

function TrDLLloader.GetEntryPoint(const Name: String): Pointer;
begin
  if FHInstance = 0 then
    Result := nil
  else
    Result := GetProcAddress(FHInstance, PChar(Name));
end;

procedure TrDLLloader.InternalGetVersionInfo;
begin
  if FLibraryName = '' then exit;
  FFileVersion := GetFileVersion(FLibraryName);
  GetVersionInfo(FVersionInfo, FLibraryName);
end;

procedure TrDLLloader.SetVersionInfo(const Value: TStrings);
begin
end;

procedure TrDLLloader.LoadDLL;
begin
  InternalGetVersionInfo;
  if CompareVersion(FFileVersion, FRequiredVersion) = -1 then begin
    ClearVersionInfo;
    raise Exception.CreateFmt(srDLLCheckVersionError, [FLibraryName, FRequiredVersion]);
  end;
  FHInstance := LoadLibrary(PChar(FLibraryName));
  if FHInstance = 0 then begin
    ClearVersionInfo;
    raise Exception.CreateFmt(srDLLLoadError, [FLibraryName]);
  end;
  try
    CheckRequired;
  except
    try
      FreeLibrary(FHInstance);
    finally
      FHInstance := 0;
    end;
    ClearVersionInfo;
    raise;
  end;
  DoAfterLoadLibrary;
end;

procedure TrDLLloader.Loaded;
begin
  inherited;
  if FActive then LoadDLL;
end;

procedure TrDLLloader.SetActive(const Value: Boolean);
begin
  if csReading in ComponentState
  then FActive := Value
  else if FActive <> Value then begin
    FActive := Value;
    FreeDLL;
    if FActive then
    try
      LoadDLL;
    except
      FActive := False;
      raise;
    end;
  end;
end;

procedure TrDLLloader.SetLibraryName(const Value: TFileName);
begin
  if csReading in ComponentState then FLibraryName := Value
  else if FLibraryName <> Value then begin
    FVersionInfo.Clear;
    if Active then raise Exception.Create(srDLLChangeNameError);
    FLibraryName := Value;
    if FLibraryName <> '' then GetVersionInfo(FVersionInfo, FLibraryName);
  end;
end;

procedure TrDLLloader.SetRequiredProcs(const Value: TStrings);
begin
  FRequiredProcs.Assign(Value);
end;

procedure TrDLLloader.SetRequiredVersion(const Value: String);
var
  MS, LS: DWORD;
begin
  if SeparateVersion(Value, MS, LS) then
    FRequiredVersion := Value
  else
    raise Exception.Create(srDLLVersionFormatError);
end;

procedure TrDLLloader.Close;
begin
  Active := False;
end;

procedure TrDLLloader.Open;
begin
  Active := True;
end;

end.
